@@ -18,10 +18,11 @@
 #define KLINEEDIT_H
 
 #include <kcompletion.h>
-#include <kcompletionbase.h>
 #include <kcompletion_export.h>
+#include <kcompletionbase.h>
 
 #include <QLineEdit>
+#include <memory>
 
 class QAction;
 class QMenu;
@@ -37,107 +38,104 @@ class KLineEditPrivate;
  * \b Detail \n
  *
  * This widget inherits from QLineEdit and implements the following
- * additional functionalities: a completion object that provides both
- * automatic and manual text completion as well as multiple match iteration
- * features, configurable key-bindings to activate these features and a
- * popup-menu item that can be used to allow the user to set text completion
- * modes on the fly based on their preference.
+ * additional functionalities:
+ * @li a completion object that provides both automatic and manual text
+ * completion as well as multiple match iteration features
+ * @li configurable key-bindings to activate these features
+ * @li a popup-menu item that can be used to allow the user to set text
+ * completion modes on the fly based on their preference
  *
- * To support these new features KLineEdit also emits a few more
- * additional signals. These are: completion( const QString& ),
- * textRotation( KeyBindingType ), and returnPressed( const QString& ).
- * The completion signal can be connected to a slot that will assist the
- * user in filling out the remaining text. The text rotation signal is
- * intended to be used to iterate through the list of all possible matches
- * whenever there is more than one match for the entered text. The
- * @c returnPressed( const QString& ) signals are the same as QLineEdit's
- * except it provides the current text in the widget as its argument whenever
- * appropriate.
+ * To support these features KLineEdit also emits a few more additional
+ * signals:
+ * @li @c completion(const QString &): this signal can be connected to
+ * a slot that will assist the user in filling out the remaining text
+ * @li @c textRotation(KeyBindingType): this signal is intended to be
+ * used to iterate through the list of all possible matches whenever
+ * there is more than one match for the entered text
+ * @li @c returnKeyPressed(const QString &): this signal provides the
+ * current text in the widget as its argument whenever appropriate (this
+ * is in addition to the @c QLineEdit::returnPressed() signal which KLineEdit
+ * inherits from QLineEdit).
  *
  * This widget by default creates a completion object when you invoke
- * the completionObject( bool ) member function for the first time or
- * use setCompletionObject( KCompletion*, bool ) to assign your own
+ * the @c completionObject(bool) member function for the first time or
+ * use @c setCompletionObject(KCompletion *, bool) to assign your own
  * completion object. Additionally, to make this widget more functional,
  * KLineEdit will by default handle the text rotation and completion
  * events internally when a completion object is created through either one
  * of the methods mentioned above. If you do not need this functionality,
- * simply use KCompletionBase::setHandleSignals( bool ) or set the
- * boolean parameter in the above functions to false.
+ * simply use @c KCompletionBase::setHandleSignals(bool) or set the boolean
+ * parameter in the above functions to false.
  *
  * The default key-bindings for completion and rotation is determined
  * from the global settings in KStandardShortcut. These values, however,
- * can be overridden locally by invoking KCompletionBase::setKeyBinding().
+ * can be overridden locally by invoking @c KCompletionBase::setKeyBinding().
  * The values can easily be reverted back to the default setting, by simply
- * calling useGlobalSettings(). An alternate method would be to default
+ * calling @c useGlobalSettings(). An alternate method would be to default
  * individual key-bindings by using setKeyBinding() with the default
  * second argument.
  *
  * If @c EchoMode for this widget is set to something other than @c QLineEdit::Normal,
  * the completion mode will always be defaulted to CompletionNone.
- * This is done purposefully to guard against protected entries such as passwords being
- * cached in KCompletion's list. Hence, if the @c EchoMode is not QLineEdit::Normal, the
- * completion mode is automatically disabled.
+ * This is done purposefully to guard against protected entries, such as
+ * passwords, being cached in KCompletion's list. Hence, if the @c EchoMode
+ * is not @c QLineEdit::Normal, the completion mode is automatically disabled.
  *
- * A read-only KLineEdit will have the same background color as a
- * disabled KLineEdit, but its foreground color will be the one used
- * for the read-write mode. This differs from QLineEdit's implementation
- * and is done to give visual distinction between the three different modes:
- * disabled, read-only, and read-write.
+ * A read-only KLineEdit will have the same background color as a disabled
+ * KLineEdit, but its foreground color will be the one used for the read-write
+ * mode. This differs from QLineEdit's implementation and is done to give visual
+ * distinction between the three different modes: disabled, read-only, and read-write.
  *
- * KLineEdit has also a password mode which depends of globals KDE settings. Use
- * KLineEdit::setPasswordMode instead of QLineEdit::echoMode property to have a password field.
- *
- * \b Usage \n
+ * @b Usage
  *
  * To enable the basic completion feature:
  *
- * \code
- * KLineEdit *edit = new KLineEdit( this );
+ * @code
+ * KLineEdit *edit = new KLineEdit(this);
  * KCompletion *comp = edit->completionObject();
- * // Connect to the return pressed signal - optional
- * connect(edit,SIGNAL(returnPressed(const QString&)),comp,SLOT(addItem(const QString&)));
- * \endcode
+ * // Connect to the Return pressed signal - optional
+ * connect(edit, &KLineEdit::returnKeyPressed, comp, [this](const QString &text) { addItem(text); });
+ * @endcode
  *
- * To use a customized completion objects or your
- * own completion object:
+ * To use a customized completion object or your own completion object:
  *
- * \code
- * KLineEdit *edit = new KLineEdit( this );
+ * @code
+ * KLineEdit *edit = new KLineEdit(this);
  * KUrlCompletion *comp = new KUrlCompletion();
- * edit->setCompletionObject( comp );
+ * edit->setCompletionObject(comp);
  * // Connect to the return pressed signal - optional
- * connect(edit,SIGNAL(returnPressed(const QString&)),comp,SLOT(addItem(const QString&)));
- * \endcode
+ * connect(edit, &KLineEdit::returnKeyPressed, comp, [this](const QString &text) { addItem(text); });
+ * @endcode
  *
  * Note if you specify your own completion object you have to either delete
  * it when you don't need it anymore, or you can tell KLineEdit to delete it
  * for you:
- * \code
- * edit->setAutoDeleteCompletionObject( true );
- * \endcode
+ * @code
+ * edit->setAutoDeleteCompletionObject(true);
+ * @endcode
  *
  * <b>Miscellaneous function calls :</b>\n
  *
- * \code
- * // Tell the widget to not handle completion and iteration automatically.
- * edit->setHandleSignals( false );
+ * @code
+ * // Tell the widget to not handle completion and iteration automatically
+ * edit->setHandleSignals(false);
  *
- * // Set your own key-bindings for a text completion mode.
- * edit->setKeyBinding( KCompletionBase::TextCompletion, Qt::End );
+ * // Set your own key-bindings for a text completion mode
+ * edit->setKeyBinding(KCompletionBase::TextCompletion, Qt::End);
  *
  * // Hide the context (popup) menu
- * edit->setContextMenuPolicy( Qt::NoContextMenu );
+ * edit->setContextMenuPolicy(Qt::NoContextMenu);
  *
- * // Default the key-bindings back to the default system settings.
+ * // Default the key-bindings back to the default system settings
  * edit->useGlobalKeyBindings();
- * \endcode
+ * @endcode
  *
- * \image html klineedit.png "KLineEdit widgets with clear-button"
+ * @image html klineedit.png "KLineEdit widgets with clear-button"
  *
  * @author Dawit Alemayehu <adawit@kde.org>
  */
 
-class KCOMPLETION_EXPORT KLineEdit : public QLineEdit, public KCompletionBase //krazy:exclude=qclasses
+class KCOMPLETION_EXPORT KLineEdit : public QLineEdit, public KCompletionBase // krazy:exclude=qclasses
 {
     friend class KComboBox;
     friend class KLineEditStyle;
@@ -158,10 +156,11 @@ class KCOMPLETION_EXPORT KLineEdit : public QLineEdit, public KCompletionBase //
 #if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 46)
     Q_PROPERTY(bool showClearButton READ isClearButtonShown WRITE setClearButtonShown)
 #endif
+#if KCOMPLETION_BUILD_DEPRECATED_SINCE(5, 83)
     Q_PROPERTY(bool passwordMode READ passwordMode WRITE setPasswordMode)
+#endif
 
 public:
-
     /**
      * Constructs a KLineEdit object with a default text, a parent,
      * and a name.
@@ -189,17 +188,17 @@ public:
     void setUrl(const QUrl &url);
 
     /**
-    * Reimplemented from KCompletionBase for internal reasons.
-    *
-    * This function is re-implemented in order to make sure that
-    * the EchoMode is acceptable before we set the completion mode.
-    *
-    * See KCompletionBase::setCompletionMode
-    */
+     * Reimplemented from KCompletionBase for internal reasons.
+     *
+     * This function is re-implemented in order to make sure that
+     * the EchoMode is acceptable before we set the completion mode.
+     *
+     * See KCompletionBase::setCompletionMode
+     */
     void setCompletionMode(KCompletion::CompletionMode mode) override;
 
     /**
-     * Disables completion modes by makeing them non-checkable.
+     * Disables completion modes by making them non-checkable.
      *
      * The context menu allows to change the completion mode.
      * This method allows to disable some modes.
@@ -242,7 +241,7 @@ public:
      *
      * @param enable If @c true, insert decoded URLs
      */
-    void setUrlDropsEnabled(bool enable);  // KF6: remove it and don't create LineEditUrlDropEventFilter by default.
+    void setUrlDropsEnabled(bool enable); // KF6: remove it and don't create LineEditUrlDropEventFilter by default.
 
     /**
      * Returns @c true when decoded URL drops are enabled
@@ -270,12 +269,12 @@ public:
     bool trapReturnKey() const;
 
     /**
-     * @returns the completion-box, that is used in completion mode
-     * CompletionPopup.
      * This method will create a completion-box if none is there, yet.
      *
      * @param create Set this to false if you don't want the box to be created
      *               i.e. to test if it is available.
+     * @returns the completion-box, that is used in completion mode
+     * CompletionPopup.
      */
     virtual KCompletionBox *completionBox(bool create = true);
 
@@ -330,7 +329,7 @@ public:
      * This will do nothing if a completion-box already exists.
      *
      * @param box The KCompletionBox to set
-    */
+     */
     void setCompletionBox(KCompletionBox *box);
 
 #if KCOMPLETION_ENABLE_DEPRECATED_SINCE(5, 0)
@@ -360,7 +359,7 @@ public:
      * This is useful for such things as location or search bars.
      *
      * @deprecated since 5.46 Use QLineEdit::setClearButtonEnabled
-     **/
+     */
     KCOMPLETION_DEPRECATED_VERSION(5, 46, "Use QLineEdit::setClearButtonEnabled(bool)")
     void setClearButtonShown(bool show);
 #endif
@@ -370,7 +369,7 @@ public:
      * @return whether or not the clear button is shown
      *
      * @deprecated since 5.46 Use QLineEdit::isClearButtonEnabled
-     **/
+     */
     KCOMPLETION_DEPRECATED_VERSION(5, 46, "Use QLineEdit::isClearButtonEnabled()")
     bool isClearButtonShown() const;
 #endif
@@ -378,7 +377,7 @@ public:
     /**
      * @return the size used by the clear button
      * @since 4.1
-     **/
+     */
     QSize clearButtonUsedSize() const;
 
     /**
@@ -391,20 +390,34 @@ public:
     void doCompletion(const QString &text);
 
 Q_SIGNALS:
-
     /**
      * Emitted whenever the completion box is activated.
      */
     void completionBoxActivated(const QString &);
 
+#if KCOMPLETION_ENABLE_DEPRECATED_SINCE(5, 81)
     /**
-     * Emitted when the user presses the return key.
+     * Emitted when the user presses the Return or Enter key.
      *
      *  The argument is the current text. Note that this
      * signal is @em not emitted if the widget's @c EchoMode is set to
      * QLineEdit::EchoMode.
+     *
+     * @deprecated since 5.81, use the KLineEdit::returnKeyPressed(const QString &) signal instead
      */
-    void returnPressed(const QString &);
+    KCOMPLETION_DEPRECATED_VERSION(5, 81, "Use the KLineEdit::returnKeyPressed(const QString &) signal instead")
+    void returnPressed(const QString &text); // clazy:exclude=overloaded-signal
+#endif
+
+    /**
+     * Emitted when the user presses the Return or Enter key.
+     *
+     * The argument is the current text. Note that this signal is @em not emitted
+     * if the widget's @c EchoMode is set to QLineEdit::EchoMode.
+     *
+     * @since 5.81
+     */
+    void returnKeyPressed(const QString &text);
 
     /**
      * Emitted when the completion key is pressed.
@@ -521,24 +534,34 @@ public Q_SLOTS:
      */
     virtual void setText(const QString &);
 
+#if KCOMPLETION_ENABLE_DEPRECATED_SINCE(5, 83)
     /**
      * @brief set the line edit in password mode.
      * this change the EchoMode according to KDE preferences.
      * @param passwordMode true to set in password mode
+     *
+     * @deprecated Since 5.83, use KPasswordLineEdit or QLineEdit::setEchoMode(QLineEdit::EchoMode).
      */
+    KCOMPLETION_DEPRECATED_VERSION(5, 83, "Use KPasswordLineEdit or QLineEdit::setEchoMode(QLineEdit::EchoMode)")
     void setPasswordMode(bool passwordMode = true);
+#endif
 
+#if KCOMPLETION_ENABLE_DEPRECATED_SINCE(5, 83)
     /**
      * @return returns true if the lineedit is set to password mode echoing
+     *
+     * @deprecated Since 5.83, see setPasswordMode(bool)
      */
+    KCOMPLETION_DEPRECATED_VERSION(5, 83, "See API docs")
     bool passwordMode() const;
+#endif
 
 protected Q_SLOTS:
 
     /**
-    * Completes the remaining text with a matching one from
-    * a given list.
-    */
+     * Completes the remaining text with a matching one from
+     * a given list.
+     */
     virtual void makeCompletion(const QString &);
 
     /**
@@ -549,69 +572,68 @@ protected Q_SLOTS:
     void userCancelled(const QString &cancelText);
 
 protected:
-
     /**
      * Reimplemented for internal reasons. API not affected.
      */
     bool event(QEvent *) override;
 
     /**
-    * Reimplemented for internal reasons. API not affected.
-    *
-    * See QLineEdit::resizeEvent().
-    */
+     * Reimplemented for internal reasons. API not affected.
+     *
+     * See QLineEdit::resizeEvent().
+     */
     void resizeEvent(QResizeEvent *) override;
 
     /**
-    * Reimplemented for internal reasons. API not affected.
-    *
-    * See QLineEdit::keyPressEvent().
-    */
+     * Reimplemented for internal reasons. API not affected.
+     *
+     * See QLineEdit::keyPressEvent().
+     */
     void keyPressEvent(QKeyEvent *) override;
 
     /**
-    * Reimplemented for internal reasons. API not affected.
-    *
-    * See QLineEdit::mousePressEvent().
-    */
+     * Reimplemented for internal reasons. API not affected.
+     *
+     * See QLineEdit::mousePressEvent().
+     */
     void mousePressEvent(QMouseEvent *) override;
 
     /**
-    * Reimplemented for internal reasons. API not affected.
-    *
-    * See QLineEdit::mouseReleaseEvent().
-    */
+     * Reimplemented for internal reasons. API not affected.
+     *
+     * See QLineEdit::mouseReleaseEvent().
+     */
     void mouseReleaseEvent(QMouseEvent *) override;
 
     /**
-    * Reimplemented for internal reasons. API not affected.
-    *
-    * See QWidget::mouseDoubleClickEvent().
-    */
+     * Reimplemented for internal reasons. API not affected.
+     *
+     * See QWidget::mouseDoubleClickEvent().
+     */
     void mouseDoubleClickEvent(QMouseEvent *) override;
 
     /**
-    * Reimplemented for internal reasons. API not affected.
-    *
-    * See QLineEdit::contextMenuEvent().
-    */
+     * Reimplemented for internal reasons. API not affected.
+     *
+     * See QLineEdit::contextMenuEvent().
+     */
     void contextMenuEvent(QContextMenuEvent *) override;
 
     /**
-    * Reimplemented for internal reasons. API not affected.
-    *
-    * See QLineEdit::createStandardContextMenu().
-    */
+     * Reimplemented for internal reasons. API not affected.
+     *
+     * See QLineEdit::createStandardContextMenu().
+     */
     QMenu *createStandardContextMenu();
 
     /**
-    * This function simply sets the lineedit text and
-    * highlights the text appropriately if the boolean
-    * value is set to true.
-    *
-    * @param text
-    * @param marked
-    */
+     * This function simply sets the lineedit text and
+     * highlights the text appropriately if the boolean
+     * value is set to true.
+     *
+     * @param text
+     * @param marked
+     */
     virtual void setCompletedText(const QString & /*text*/, bool /*marked*/);
 
     /**
@@ -622,19 +644,13 @@ protected:
 
     /**
      * Whether in current state text should be auto-suggested
-    */
+     */
     bool autoSuggest() const;
 
     void paintEvent(QPaintEvent *ev) override;
 
 private:
-    const QScopedPointer<KLineEditPrivate> d_ptr;
-
-    Q_PRIVATE_SLOT(d_func(), void _k_textChanged(const QString &))
-    Q_PRIVATE_SLOT(d_func(), void _k_completionMenuActivated(QAction *))
-    Q_PRIVATE_SLOT(d_func(), void _k_tripleClickTimeout())
-    Q_PRIVATE_SLOT(d_func(), void _k_restoreSelectionColors())
-    Q_PRIVATE_SLOT(d_func(), void _k_completionBoxTextChanged(const QString &))
+    std::unique_ptr<KLineEditPrivate> const d_ptr;
 };
 
 #endif
